# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-01-15 15:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0028_make-subparser-nonnullable'),
    ]

    operations = [
        migrations.AddField(
            model_name='scriptparametergroup',
            name='new_script_version',
            field=models.ManyToManyField(related_name='new_script_version_group', to='wooey.ScriptVersion'),
        ),
        migrations.AddField(
            model_name='scriptparser',
            name='new_script_version',
            field=models.ManyToManyField(related_name='new_script_version', to='wooey.ScriptVersion'),
        ),
    ]
