# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-01-15 15:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0031_remove-sv'),
    ]

    operations = [
        migrations.RenameField(
            model_name='scriptparametergroup',
            old_name='new_script_version',
            new_name='script_version'
        ),
        migrations.RenameField(
            model_name='scriptparser',
            old_name='new_script_version',
            new_name='script_version'
        )
    ]
