# Generated by Django 2.1.15 on 2020-04-12 13:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0041_change-script-param-to-textfield'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scriptversion',
            name='script',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='script_version', to='wooey.Script'),
        ),
    ]
